package me.mrletsplay.videobase.util;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.concurrent.TimeUnit;

import javax.imageio.ImageIO;

import me.mrletsplay.videobase.VideoBase;
import me.mrletsplay.videobase.library.Video;

public class ThumbnailCreator {

	public static BufferedImage createThumbnail(Video video) {
		Path cacheDir = Path.of(VideoBase.getConfig().getCachePath()).resolve("thumbnails").toAbsolutePath();
		Path thumbnailFile = cacheDir.resolve(video.getId() + ".png");
		try {
			Files.createDirectories(cacheDir);
			if(Files.exists(thumbnailFile)) {
				try(InputStream in = Files.newInputStream(thumbnailFile)) {
					return ImageIO.read(in);
				}
			}
		} catch (IOException e) {
			VideoBase.LOGGER.warn("Failed to load thumbnail", e);
			return null;
		}

		ProcessBuilder ffmpegBuilder = new ProcessBuilder("ffmpeg",
			"-i", video.getPath().toAbsolutePath().toString(),
			"-vf", "thumbnail",
			"-vf", "scale=960:540:force_original_aspect_ratio=decrease,pad=960:540:-1:-1:color=black",
			"-frames:v", "1",
			"-f", "image2pipe",
			"-vcodec", "png",
			"-"
		);

		try {
			Process ffmpeg = ffmpegBuilder.start();
			long start = System.currentTimeMillis();

			ByteArrayOutputStream bOut = new ByteArrayOutputStream();
			byte[] buf = new byte[1024];
			int len;
			while(System.currentTimeMillis() - start < 1000
				&& (len = ffmpeg.getInputStream().read(buf)) > 0) {
				bOut.write(buf, 0, len);
			}

			ffmpeg.waitFor(10, TimeUnit.SECONDS);
			if(ffmpeg.isAlive()) {
				VideoBase.LOGGER.warn("ffmpeg didn't exit after 10 seconds, destroying");
				ffmpeg.destroy();

				if(!ffmpeg.waitFor(10, TimeUnit.SECONDS)) {
					VideoBase.LOGGER.warn("ffmpeg didn't exit 10 seconds after destroying, destroying forcibly");
					ffmpeg.destroyForcibly();
				}

				return null;
			}

			BufferedImage thumbnail = ImageIO.read(new ByteArrayInputStream(bOut.toByteArray()));
			try(OutputStream out = Files.newOutputStream(thumbnailFile)) {
				ImageIO.write(thumbnail, "PNG", out);
			}

			return thumbnail;
		} catch (InterruptedException | IOException e) {
			VideoBase.LOGGER.warn("Failed to create thumbnail", e);
			return null;
		}

	}


}
